@@ -8,7 +8,7 @@ from resotolib.graph import Graph, sanitize
 from resotolib.logger import log, setup_logger
 from resotolib.args import ArgumentParser
 from argparse import Namespace
-from typing import List, Optional, Callable
+from typing import List, Optional, Callable, Type, Dict, Any
 from resotolib.config import Config, RunningConfig
 
 TaskId = str
@@ -21,12 +21,12 @@ class Collector:
 
     def collect_and_send(
         self,
-        collectors: List[BaseCollectorPlugin],
-        post_collectors: List[BasePostCollectPlugin],
+        collectors: List[Type[BaseCollectorPlugin]],
+        post_collectors: List[Type[BasePostCollectPlugin]],
         task_id: str,
     ) -> None:
-        def collect(collectors: List[BaseCollectorPlugin]) -> Graph:
-            graph = Graph(root=GraphRoot("root", {}))
+        def collect(collectors: List[Type[BaseCollectorPlugin]]) -> Optional[Graph]:
+            graph = Graph(root=GraphRoot(id="root", tags={}))
 
             max_workers = (
                 len(collectors)
@@ -35,8 +35,10 @@ class Collector:
             )
             if max_workers == 0:
                 log.error("No workers configured or no collector plugins loaded - skipping collect")
-                return
+                return None
             pool_args = {"max_workers": max_workers}
+            pool_executor: Type[futures.Executor]
+            collect_args: Dict[str, Any]
             if self._config.resotoworker.fork_process:
                 pool_args["mp_context"] = multiprocessing.get_context("spawn")
                 pool_args["initializer"] = resotolib.proc.initializer
@@ -67,22 +69,83 @@ class Collector:
             sanitize(graph)
             return graph
 
+        def post_collect(graph: Graph, post_collectors: List[Type[BasePostCollectPlugin]]) -> Graph:
+            if len(post_collectors) == 0:
+                log.error("No workers configured or no collector plugins loaded - skipping collect")
+                return graph
+            pool_args: Dict[str, Any] = {"max_workers": 1}
+            pool_executor: Type[futures.Executor]
+            collect_args: Dict[str, Any] = {}
+            if self._config.resotoworker.fork_process:
+                pool_args["mp_context"] = multiprocessing.get_context("spawn")
+                pool_args["initializer"] = resotolib.proc.initializer
+                pool_executor = futures.ProcessPoolExecutor
+                collect_args = {
+                    "args": ArgumentParser.args,
+                    "running_config": self._config.running_config,
+                }
+            else:
+                pool_executor = futures.ThreadPoolExecutor
+
+            with pool_executor(**pool_args) as executor:
+                for post_collector in post_collectors:
+                    future = executor.submit(run_post_collect_plugin, post_collector, graph, **collect_args)
+                    try:
+                        new_graph = future.result(Config.resotoworker.timeout)
+                    except TimeoutError as e:
+                        log.exception(f"Unhandled exception in {post_collector}: {e} - ignoring plugin")
+                        continue
+                    except Exception as e:
+                        log.exception(f"Unhandled exception in {post_collector}: {e} - ignoring plugin")
+                        continue
+
+                    if new_graph is None:
+                        continue
+                    graph = new_graph
+
+                sanitize(graph)
+                return graph
+
         collected = collect(collectors)
 
-        for post_collector_class in post_collectors:
-            try:
-                instance = post_collector_class()
-                collected = instance.post_collect(collected)
-            except Exception as e:
-                log.warn(f"collector {post_collector_class.name} was not able to collect external edges: {e}")
+        if collected:
+            collected = post_collect(collected, post_collectors)
+            self._send_to_resotocore(collected, task_id)
 
-        self._send_to_resotocore(collected, task_id)
+
+def run_post_collect_plugin(
+    post_collector_plugin: Type[BasePostCollectPlugin],
+    graph: Graph,
+    args: Optional[Namespace] = None,
+    running_config: Optional[RunningConfig] = None,
+) -> Optional[Graph]:
+    try:
+        post_collector: BasePostCollectPlugin = post_collector_plugin()
+
+        if args is not None:
+            ArgumentParser.args = args  # type: ignore
+            setup_logger("resotoworker")
+        if running_config is not None:
+            Config.running_config.apply(running_config)
+
+        log.debug(f"starting new post-collect process for {post_collector.name}")
+        start_time = time()
+        post_collector.post_collect(graph)
+        elapsed = time() - start_time
+        if not graph.is_dag_per_edge_type():
+            log.error(f"Graph of plugin {post_collector.name} is not acyclic - ignoring plugin results")
+            return None
+        log.info(f"Collector of plugin {post_collector.name} finished in {elapsed:.4f}s")
+        return graph
+    except Exception as e:
+        log.exception(f"Unhandled exception in {post_collector_plugin}: {e} - ignoring plugin")
+        return None
 
 
 def collect_plugin_graph(
-    collector_plugin: BaseCollectorPlugin,
-    args: Namespace = None,
-    running_config: RunningConfig = None,
+    collector_plugin: Type[BaseCollectorPlugin],
+    args: Optional[Namespace] = None,
+    running_config: Optional[RunningConfig] = None,
 ) -> Optional[Graph]:
     try:
         collector: BaseCollectorPlugin = collector_plugin()
@@ -90,7 +153,7 @@ def collect_plugin_graph(
         resotolib.proc.set_thread_name(collector_name)
 
         if args is not None:
-            ArgumentParser.args = args
+            ArgumentParser.args = args  # type: ignore
             setup_logger("resotoworker")
         if running_config is not None:
             Config.running_config.apply(running_config)
